@@ -1,4 +1,4 @@
-/* $Id: ml_togl.c,v 1.14 2004/11/02 07:03:34 garrigue Exp $ */
+/* $Id: ml_togl.c,v 1.16 2006/03/23 06:01:55 garrigue Exp $ */
 
 #ifdef _WIN32
 #include <wtypes.h>
@@ -32,6 +32,15 @@ int TOGLenum_val(value tag)
     invalid_argument ("Unknown Togl tag");
 }
 
+/* Avoid direct use of stderr */
+void togl_prerr(const char *msg)
+{
+    value ml_msg = copy_string(msg);
+    value *prerr = caml_named_value("togl_prerr");
+    if (!prerr) caml_failwith(msg);
+    caml_callback_exn(*prerr, ml_msg);
+}
+
 CAMLprim value ml_Togl_Init (value unit)  /* ML */
 {
     value *interp = caml_named_value("cltclinterp");
@@ -139,3 +148,9 @@ CAMLprim value ml_Togl_DumpToEpsFile (value togl, value filename, value rgb)
                           "Dump to EPS file failed");
     return Val_unit;
 }
+
+#if 0 && defined(_WIN32) && !defined(CAML_DLL) && (WINVER < 0x0500)
+/* VC7 or later, building with pre-VC7 runtime libraries */
+long _ftol( double ); /* defined by VC6 C libs */
+long _ftol2( double dblSource ) { return _ftol( dblSource ); }
+#endif
