@@ -1,4 +1,4 @@
-/* $Id: ml_gl.c,v 1.47 2005/10/14 13:35:32 garrigue Exp $ */
+/* $Id: ml_gl.c,v 1.51 2007/04/13 02:48:43 garrigue Exp $ */
 
 #ifdef _WIN32
 #include <wtypes.h>
@@ -9,6 +9,10 @@
 #else
 #include <GL/gl.h>
 #endif
+#ifdef HAS_GLEXT_H
+#include <GL/glext.h>
+#undef GL_VERSION_1_3
+#endif
 #include <caml/misc.h>
 #include <caml/mlvalues.h>
 #include <caml/callback.h>
@@ -179,6 +183,9 @@ ML_5 (glEvalMesh2, GLenum_val, Int_val, Int_val, Int_val, Int_val)
 ML_1 (glEvalPoint1, Int_val)
 ML_2 (glEvalPoint2, Int_val, Int_val)
 
+
+ML_3 (glFeedbackBuffer, Int_val, GLenum_val, (GLfloat*)Addr_raw)
+
 CAMLprim value ml_glFog (value param) /* ML */
 {
     float params[4];
@@ -477,6 +484,7 @@ CAMLprim value ml_glPixelTransfer (value param)
 
 ML_2 (glPixelZoom, Float_val, Float_val)
 ML_1 (glPointSize, Float_val)
+ML_2 (glPolygonOffset, Float_val, Float_val)
 ML_2 (glPolygonMode, GLenum_val, GLenum_val)
 ML_1 (glPolygonStipple, (unsigned char *)Byte_raw)
 ML_0 (glPopAttrib)
@@ -662,25 +670,13 @@ CAMLprim value ml_glTexParameter (value target, value param)
     }
     return Val_unit;
 }
-    
-CAMLprim value ml_glGenTexture (value unit)
-{
-    GLuint id;
-    glGenTextures(1,&id);
-    return copy_int32(id);
-}
 
-CAMLprim value ml_glBindTexture (value target, value texture_id)
-{
-    GLenum targ = GLenum_val(target);
-    GLuint id = Int32_val(texture_id);
-    glBindTexture(targ,id);
-    return Val_unit;
-}
+ML_2 (glGenTextures, Int_val, Int_raw)
+ML_2 (glBindTexture, GLenum_val, Nativeint_val)
 
 CAMLprim value ml_glDeleteTexture (value texture_id)
 {
-    GLuint id = Int32_val(texture_id);
+    GLuint id = Nativeint_val(texture_id);
     glDeleteTextures(1,&id);
     return Val_unit;
 }
