@@ -1,4 +1,4 @@
-/* $Id: ml_raw.c,v 1.14 2004/12/02 02:01:16 garrigue Exp $ */
+/* $Id: ml_raw.c,v 1.16 2007/04/13 02:48:43 garrigue Exp $ */
 
 #include <string.h>
 #include <caml/misc.h>
@@ -420,6 +420,40 @@ CAMLprim value ml_raw_set_lo (value raw, value pos, value data)  /* ML */
     return Val_unit;
 }
 
+CAMLprim value ml_raw_get_long (value raw, value pos)  /* ML */
+{
+    long i = Long_val(pos);
+
+    check_size (raw,i,"Raw.get_long");
+    switch (Kind_raw(raw)) {
+    case MLTAG_int:
+    case MLTAG_uint:
+        return copy_nativeint (Int_raw(raw)[i]);
+    case MLTAG_long:
+    case MLTAG_ulong:
+        return copy_nativeint (Long_raw(raw)[i]);
+    }
+    return Val_unit;
+}
+
+CAMLprim value ml_raw_set_long (value raw, value pos, value data)  /* ML */
+{
+    long i = Long_val(pos);
+
+    check_size (raw,i,"Raw.set_long");
+    switch (Kind_raw(raw)) {
+    case MLTAG_int:
+    case MLTAG_uint:
+	Int_raw(raw)[i] = Nativeint_val(data);
+	break;
+    case MLTAG_long:
+    case MLTAG_ulong:
+	Long_raw(raw)[i] = Nativeint_val(data);
+	break;
+    }
+    return Val_unit;
+}
+
 CAMLprim value ml_raw_alloc (value kind, value len)  /* ML */
 {
     CAMLparam0();
@@ -468,5 +502,6 @@ CAMLprim value ml_raw_free_static (value raw)  /* ML */
     Base_raw(raw) = Val_unit;
     Size_raw(raw) = Val_unit;
     Offset_raw(raw) = Val_unit;
+    Static_raw(raw) = Val_false;
     return Val_unit;
 }
